Index: src/libpocketsphinx/dict.c
===================================================================
--- src/libpocketsphinx/dict.c	(revision 8731)
+++ src/libpocketsphinx/dict.c	(working copy)
@@ -734,8 +734,8 @@
     }
 
     /* For the moment, no single phone new word... */
-    if (pronoun_len < 2) {
-        E_ERROR("Pronunciation string too short\n");
+    if (pronoun_len < 1) {
+        E_ERROR("Pronunciation string is empty\n");
         return (0);
     }
 
@@ -743,43 +743,54 @@
     {
         char *p = strrchr(word_str, '(');
         if (p && (word_str[strlen(word_str) - 1] == ')')) {
-            *p = '\0';
-            if (hash_table_lookup_int32(dict->dict, word_str, &idx)) {
-                *p = '(';
-                E_ERROR("Base word missing for %s\n", word_str);
-                return 0;
-            }
-            *p = '(';
-            basewid = (long)idx;
-        }
-        else
-            basewid = -1;
+    	    *p = '\0';
+    	    if (hash_table_lookup_int32(dict->dict, word_str, &idx)) {
+        	*p = '(';
+            	E_ERROR("Base word missing for %s\n", word_str);
+            	return 0;
+    	    }
+        *p = '(';
+        basewid = (long)idx;
+	} else {
+    	    basewid = -1;
+	}
     }
 
-    /* Parse pron; for the moment, the boundary diphones must be already known... */
-    i = 0;
-    sprintf(triphoneStr, "%s(%%s,%s)b", phone[i], phone[i + 1]);
-    if (hash_table_lookup_int32(dict->lcHT, triphoneStr, &idx) < 0) {
-        E_ERROR("Unknown left diphone '%s'\n", triphoneStr);
-        return (0);
-    }
-    triphone_ids[i] = idx;
+    if (pronoun_len >= 2) {
 
-    for (i = 1; i < pronoun_len - 1; i++) {
-        triphone_ids[i] = bin_mdef_phone_id(dict->mdef,
-                                            dict_ciphone_id(dict, phone[i]),
-                                            dict_ciphone_id(dict, phone[i-1]),
-                                            dict_ciphone_id(dict, phone[i+1]),
-                                            WORD_POSN_INTERNAL);
-        triphone_ids[i] = bin_mdef_pid2ssid(dict->mdef, triphone_ids[i]);
+	/* Parse pron; for the moment, the boundary diphones must be already known... */
+        i = 0;
+	sprintf(triphoneStr, "%s(%%s,%s)b", phone[i], phone[i + 1]);
+        if (hash_table_lookup_int32(dict->lcHT, triphoneStr, &idx) < 0) {
+	    E_ERROR("Unknown left diphone '%s'\n", triphoneStr);
+            return (0);
+	}
+        triphone_ids[i] = idx;
+
+	for (i = 1; i < pronoun_len - 1; i++) {
+            triphone_ids[i] = bin_mdef_phone_id(dict->mdef,
+                    	                        dict_ciphone_id(dict, phone[i]),
+                    		                dict_ciphone_id(dict, phone[i-1]),
+                	                        dict_ciphone_id(dict, phone[i+1]),
+        	                                WORD_POSN_INTERNAL);
+    	    triphone_ids[i] = bin_mdef_pid2ssid(dict->mdef, triphone_ids[i]);
+	}
+    
+        sprintf(triphoneStr, "%s(%s,%%s)e", phone[i], phone[i - 1]);
+	if (hash_table_lookup_int32(dict->rcHT, triphoneStr, &idx) < 0) {
+        	E_ERROR("Unknown right diphone '%s'\n", triphoneStr);
+    	    return (0);
+	}
+        triphone_ids[i] = idx;
     }
 
-    sprintf(triphoneStr, "%s(%s,%%s)e", phone[i], phone[i - 1]);
-    if (hash_table_lookup_int32(dict->rcHT, triphoneStr, &idx) < 0) {
-        E_ERROR("Unknown right diphone '%s'\n", triphoneStr);
-        return (0);
+    /*
+     * Add single phone words without context
+     */
+    if (pronoun_len == 1) {
+        triphone_ids[0] = dict_ciphone_id(dict,phone[0]);
+        triphone_ids[0] = bin_mdef_pid2ssid(dict->mdef,triphone_ids[0]);
     }
-    triphone_ids[i] = idx;
 
     /*
      * Set up dictionary entry.  Free the existing attributes (where applicable) and
Index: src/libpocketsphinx/fsg_search.c
===================================================================
--- src/libpocketsphinx/fsg_search.c	(revision 8731)
+++ src/libpocketsphinx/fsg_search.c	(working copy)
@@ -201,6 +201,30 @@
 
 
 static int
+fsg_search_clean_oov(fsg_search_t *fsgs, fsg_model_t *fsg)
+{
+    dict_t *dict;
+    int i;
+
+    dict = ps_search_dict(fsgs);
+
+    for (i = 0; i < fsg_model_n_word(fsg); ++i) {
+        char const *word;
+        int32 wid;
+        char buf[4] = "SIL";
+        
+        word = fsg_model_word_str(fsg, i);
+        wid = dict_to_id(dict, word);
+        if (wid == NO_WORD) {
+            E_WARN("Word %s is missing in the dictionary, adding with a SIL pron\n", word);
+	    dict_add_word (dict, word, buf);
+    	}
+    }
+    
+    return 0;
+}
+
+static int
 fsg_search_add_silences(fsg_search_t *fsgs, fsg_model_t *fsg)
 {
     dict_t *dict;
@@ -282,6 +306,8 @@
 {
     if (name == NULL)
         name = fsg_model_name(fsg);
+    
+    fsg_search_clean_oov(fsgs, fsg);
 
     /* Add silence transitions and alternate words. */
     if (cmd_ln_boolean_r(ps_search_config(fsgs), "-fsgusefiller")
